/*
 * Copyright © 2021 Keith Packard <keithp@keithp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 */

#include <ao.h>

void
ao_i2c_bit_get(void);

void
ao_i2c_bit_put(void);

bool
ao_i2c_bit_start(uint8_t addr);

bool
ao_i2c_bit_restart(uint8_t addr);

void
ao_i2c_bit_stop(void);

void
ao_i2c_bit_send(void *block, uint16_t len);

void
ao_i2c_bit_recv(void *block, uint16_t len);

void
ao_i2c_bit_init(void);
